/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.web.listener;

import ch.hortis.sonar.core.batch.Batch;
import ch.hortis.sonar.jpa.JPAUtil;
import ch.hortis.sonar.model.JdbcData;
import ch.hortis.sonar.model.JdbcData.JDBCDialect;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NameNotFoundException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.persistence.PersistenceException;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.sql.DataSource;

import org.apache.commons.lang.StringUtils;

import java.sql.SQLException;

/**
 * Servlet context listener to startup/stop sonar core
 */
public class SonarCoreStartupListener implements ServletContextListener {


  public void contextInitialized( ServletContextEvent ctxEvent ) {
    Context ctx;
    try {
      ctx = new InitialContext();
    } catch (NamingException ex) {
      ctxEvent.getServletContext().log( "Naming exception occured during JNDI initial context creation", ex );
      return;
    }
    
    String url = lookup( ctx, "java:comp/env/sonar/jdbc-url", null );
    String driver = lookup( ctx, "java:comp/env/sonar/jdbc-driver", null );
    String password = lookup( ctx, "java:comp/env/sonar/db-password", null );
    String username = lookup( ctx, "java:comp/env/sonar/db-username", null );
    
    String datasource = JdbcData.DEFAULT_SONAR_DS_JNDI_NAME;
    String jdbcDialect = lookup( ctx, "java:comp/env/sonar/jdbc-dialect", null );
    if ( jdbcDialect == null ) {
      jdbcDialect = autodetectJDBCDialect( ctxEvent, ctx, datasource );
      SonarContext.getInstance().put( SonarContext.JDBC_DIALECT_CONTEXT_KEY, jdbcDialect );
    }
    
    try {
      populateSonarContext( ctx );
    } catch ( NameNotFoundException ex ) {
      // no java:comp/env/sonar found.. not an issue
    } catch ( NamingException ex ) {
      ctxEvent.getServletContext().log( "Naming exception occured during java:comp/env/sonar lookups", ex );
    }

    if ( url != null && driver != null && password != null && username != null ) {
      Batch.getInstance().start(url, driver, username, password);
    } else if ( jdbcDialect != null ) {
      Batch.getInstance().start(datasource, jdbcDialect);
    } else {
      ctxEvent.getServletContext().log( "Missing JDBC settings" );
    }

    try {
      ctx.close();
    } catch (NamingException e) {
      // should log a warning
    }

  }
  
  private void populateSonarContext(Context ctx) throws NamingException{
    SonarContext sonarContext = SonarContext.getInstance();
    NamingEnumeration<NameClassPair> rorEnv = ctx.list( "java:comp/env/sonar" );
    while ( rorEnv.hasMoreElements() ) {
      NameClassPair pair = rorEnv.nextElement();
      Object val = ctx.lookup( "java:comp/env/sonar/" + pair.getName() );
      if ( val == null ) val = "";
      sonarContext.put( pair.getName(), val.toString() );
    }
  }
  
  private String autodetectJDBCDialect(ServletContextEvent ctxEvent, Context ctx, String datasource) {
    DataSource ds = lookup( ctx, datasource, null );
    if ( ds != null ) {
      try {
        String jdbcURL = ds.getConnection().getMetaData().getURL();
        return JPAUtil.getHibernateDialectName( jdbcURL );
      } catch ( SQLException ex ) {
        ctxEvent.getServletContext().log( "SQLException occured during autodetectJDBCDialect", ex );
      } catch ( PersistenceException ex ) {
        String errorMessage = "Unable to autodetect JDBC dialect to use, please setup the " +
        		"JNDI java:comp/env/sonar/jdbc-dialect var with one of the following value : ";
        JDBCDialect[] dialects = JDBCDialect.values();
        for (JDBCDialect dialect : dialects) {
          errorMessage += dialect.name() + ",";
        }
        errorMessage = StringUtils.chop(errorMessage);
        ctxEvent.getServletContext().log( errorMessage, ex );
      }
    } else {
      ctxEvent.getServletContext().log( "Unable to lookup datasource at " + datasource );
    }
    return null;
  }

  public void contextDestroyed( ServletContextEvent ctxEvent ) {
    Batch.getInstance().shutdown();
    SonarContext.getInstance().destroy();
  }
  
  private <T> T lookup( Context ctx, String key, Object defaultValue ) {
      Object value;
      try {
        value = ctx.lookup( key );
      } catch ( NamingException e ) {
        value = defaultValue;
      }
      return (T)value;
  }
}